<?php
/**
 * This code is licensed under the BSD 3-Clause License.
 *
 * Copyright (c) 2017, Maks Rafalko
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 * * Neither the name of the copyright holder nor the names of its
 *   contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

declare(strict_types=1);

namespace Infection\Tests\Process\Runner;

use function implode;
use Infection\Process\Runner\InitialTestsFailed;
use Infection\TestFramework\AbstractTestFrameworkAdapter;
use PHPUnit\Framework\TestCase;
use Symfony\Component\Process\Process;

final class InitialTestsFailedTest extends TestCase
{
    public function test_log_initial_tests_do_not_pass(): void
    {
        $process = $this->createMock(Process::class);
        $process->expects($this->once())->method('getExitCode')->willReturn(0);
        $process->expects($this->once())->method('getOutput')->willReturn('output string');
        $process->expects($this->once())->method('getErrorOutput')->willReturn('error string');
        $process->expects($this->once())->method('getCommandLine')->willReturn('vendor/bin/phpunit --order=random');

        $testFrameworkAdapter = $this->createMock(AbstractTestFrameworkAdapter::class);
        $testFrameworkAdapter->expects($this->once())->method('getName')->willReturn('phpunit');
        $testFrameworkAdapter->expects($this->once())->method('getInitialTestsFailRecommendations')->willReturn('-');

        $error = implode("\n", [
            'Project tests must be in a passing state before running Infection.',
            '-',
            'phpunit reported an exit code of 0.',
            'Refer to the phpunit\'s output below:',
            'STDOUT:',
            'output string',
            'STDERR:',
            'error string',
        ]);

        $exception = InitialTestsFailed::fromProcessAndAdapter($process, $testFrameworkAdapter);

        $this->assertSame($error, $exception->getMessage());
    }
}
